﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SelectRectDemoCSharp
{
    public partial class fMain : Form

    {
        int status;
        // Status indicates the action that is currently happening
        // 0 - Inactive
        // 1 - Waiting to draw a rectangle (button has been clicked)
        // 2 - Drawing a rectangle
        //     (one corner selected, mouse is moving the opposite corner)
        // 3 - Rectangle is selected and available for resizing
        // 4 - Rectangle is being resized
        // 5 - Rectangle is being moved

        int x1, y1, x2, y2;

        struct ResizeGrab
        {
            public int x;
            public int y;
            public Boolean active;
        }

        ResizeGrab[] resizeGrabs = new ResizeGrab[8];
        // This array contains details of up to 8 resize 'Grabs' that can be
        // used to resize the rectangle.
        // Elements 0 to 3 are the corner grabs, which always exist
        // 0 - Top left
        // 1 - Top right
        // 2 - Bottom left
        // 3 - Bottom right
        // Elements 4 to 7 are the side grabs, which may not exist if there
        // is not enough room for them.
        // 4 - Top
        // 5 - Bottom
        // 6 - Left
        // 7 - Right

        int grabNumber;
        // grabNumber indicates the ResizeGrab (0 to 7) which is being used.
        // This can also take the value 8, which indicates the image is being moved.

        int xOffset, yOffset;


        public fMain()
        {
            InitializeComponent();
        }

        private void cmdClear_Click(object sender, EventArgs e)
        {
            selectClear();

            cmdSelect.Enabled = true;
            cmdLoad.Enabled = true;
            cmdCrop.Enabled = false;
            cmdClear.Enabled = false;
        }

        private void cmdCrop_Click(object sender, EventArgs e)
        {
            // Clear the selection before cropping
            axImageBox1.SelectRectangle(x1, y1, x2, y2);
            selectClear();
            axImageBox1.CropToSelection();

            cmdSelect.Enabled = true;
            cmdLoad.Enabled = true;
            cmdCrop.Enabled = false;
            cmdClear.Enabled = false;
        }

        private void cmdLoad_Click(object sender, EventArgs e)
        {
            dlgOpen.Title = "Load Image";
            dlgOpen.Filter = "Bitmap Files (*.bmp)|*.bmp|JPEG Files (*.jpg, *.jpeg)|*.jpg;*.jpeg" +
             "|PCX Files (*.pcx)|*.pcx|PNG Files (*.png)|*.png" +
             "|GIF Files (*.gif)|*.gif" +
             "|Wireless Bitmap Files (*.wbm, *.wbmp)|*.wbm;*.wbmp" +
             "|Adobe Photoshop Files (*.psd)|*.psd" +
             "|TIFF Files (*.tif, *.tiff)|*.tif;*.tiff" +
             "|All Image Files|*.bmp;*.jpg;*.jpeg;*.pcx;*.png;*.gif;*.wbm;*.wbmp;*.psd;*.tif;*.tiff";
            dlgOpen.FilterIndex = 9;
            if (dlgOpen.ShowDialog() != DialogResult.Cancel)
            {
                axImageBox1.LoadFromFile(dlgOpen.FileName);
                if (axImageBox1.AutoScale())
                    MessageBox.Show("Image is too large and has been reduced in size", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                cmdSelect.Enabled = true;
                cmdCrop.Enabled = false;
                cmdClear.Enabled = false;
            }
        }

        private void cmdSelect_Click(object sender, EventArgs e)
        {
            // Clear the existing rectangle if selected already
            if (status == 3)
                selectClear();

            // Set status to indicate that next mouse click will select the first corner
            status = 1;

            // Disable all buttons
            cmdSelect.Enabled = false;
            cmdLoad.Enabled = false;
            cmdCrop.Enabled = false;
            cmdClear.Enabled = false;
        }

        private void fMain_Load(object sender, EventArgs e)
        {
            axImageBox1.NewImage(axImageBox1.Width, axImageBox1.Height);
            status = 0;

            cmdSelect.Enabled = true;
            cmdLoad.Enabled = true;
            cmdCrop.Enabled = false;
            cmdClear.Enabled = false;

            // This PenMode means that a line can be erased by re-drawing on top of it
            axImageBox1.PenMode = csXImageTrial.TxPenMode.pmNotXor;

            // Any PenStyle can be chosen, e.g. Dashed Lines
            axImageBox1.PenStyle = csXImageTrial.TxPenStyle.psDash;
        }

        private void axImageBox1_OnMouseDown(object sender, AxcsXImageTrial.IImageBoxEvents_OnMouseDownEvent e)
        {
            switch (status)
            {
                case 1:
                    // Record the start coordinates of the line
                    x1 = e.x;
                    y1 = e.y;
                    x2 = e.x;
                    y2 = e.y;

                    // Reset status to indicate 2nd corner is now selected
                    status = 2;
                    break;
                case 3:
                    drawResizeGrabs();
                    if (grabNumber == 8)
                    {
                        // Rectangle will be moved
                        status = 5;

                        // Record the distance from the mouse pointer to the top left corner
                        xOffset = e.x - x1;
                        yOffset = e.y - y1;

                        // Change to the 'closed hand' cursor
                        fMain.ActiveForm.Cursor = new Cursor(System.AppDomain.CurrentDomain.BaseDirectory + "handgrab.cur");

                    }
                    else
                        // Rectangle will be resized
                        status = 4;
                    break;
            }
        }

        private void axImageBox1_OnMouseMove(object sender, AxcsXImageTrial.IImageBoxEvents_OnMouseMoveEvent e)
        {
            int w, h;

            switch (status)
            {
                case 1:
                    // Use the crosshair cursor when drawing
                    fMain.ActiveForm.Cursor = Cursors.Cross;
                    break;
                case 2:
                    // Use the crosshair cursor when drawing
                    fMain.ActiveForm.Cursor = Cursors.Cross;

                    // Erase the previous rectangle
                    // (note the PenMode setting from fMain_Load)
                    axImageBox1.Rectangle(x1, y1, x2 + 1, y2 + 1);

                    // Set the new 2nd corner coordinates and draw a new rectangle
                    x2 = e.x;
                    y2 = e.y;
                    axImageBox1.Rectangle(x1, y1, x2 + 1, y2 + 1);
                    break;
                case 3:
                    grabNumber = -1;
                    // Check whether the mouse pointer is over one of the grabs
                    for (int i = 0; i < 8; i++)
                    {
                        if ((Math.Abs(resizeGrabs[i].x - e.x) < 3) && (Math.Abs(resizeGrabs[i].y - e.y) < 3) &&
                            resizeGrabs[i].active)
                        {
                            grabNumber = i;
                        }
                    }
                    // Set the appropriate pointer for resizing and moving
                    switch (grabNumber)
                    {
                        case -1:
                            if ((e.x >= x1) && (e.x < x2) && (e.y >= y1) && (e.y < y2))
                            {
                                fMain.ActiveForm.Cursor = new Cursor(System.AppDomain.CurrentDomain.BaseDirectory + "handflat.cur");
                                grabNumber = 8;
                            }
                            else
                                fMain.ActiveForm.Cursor = Cursors.Default;
                            break;
                        case 0:
                        case 1:
                        case 2:
                        case 3:
                            fMain.ActiveForm.Cursor = Cursors.SizeAll;
                            break;
                        case 4:
                        case 5:
                            fMain.ActiveForm.Cursor = Cursors.SizeNS;
                            break;
                        case 6:
                        case 7:
                            fMain.ActiveForm.Cursor = Cursors.SizeWE;
                            break;
                    }
                    break;
                case 4:
                    // Erase the existing rectangle
                    axImageBox1.Rectangle(x1, y1, x2 + 1, y2 + 1);

                    // Reset the coordinates depending on the resize grab being used
                    switch (grabNumber)
                    {
                        case 0:
                            x1 = e.x;
                            y1 = e.y;
                            break;
                        case 1:
                            x2 = e.x;
                            y1 = e.y;
                            break;
                        case 2:
                            x1 = e.x;
                            y2 = e.y;
                            break;
                        case 3:
                            x2 = e.x;
                            y2 = e.y;
                            break;
                        case 4:
                            y1 = e.y;
                            break;
                        case 5:
                            y2 = e.y;
                            break;
                        case 6:
                            x1 = e.x;
                            break;
                        case 7:
                            x2 = e.x;
                            break;
                    }
                    // Redraw the rectangle
                    axImageBox1.Rectangle(x1, y1, x2 + 1, y2 + 1);
                    break;
                case 5:
                    // Erase the existing rectangle
                    axImageBox1.Rectangle(x1, y1, x2 + 1, y2 + 1);

                    // Reset the coordinates
                    w = x2 - x1;
                    h = y2 - y1;
                    x1 = e.x - xOffset;
                    x2 = x1 + w;
                    y1 = e.y - yOffset;
                    y2 = y1 + h;

                    // Redraw the rectangle
                    axImageBox1.Rectangle(x1, y1, x2 + 1, y2 + 1);
                    break;
            }
        }

        private void axImageBox1_OnMouseUp(object sender, AxcsXImageTrial.IImageBoxEvents_OnMouseUpEvent e)
        {
            if ((status == 2) || (status == 4) || (status == 5))
            {
                status = 3;
                cmdSelect.Enabled = true;
                cmdLoad.Enabled = true;
                cmdCrop.Enabled = true;
                cmdClear.Enabled = true;
                calcResizeGrabs();
                drawResizeGrabs();
            }
        }

        private void fMain_MouseMove(object sender, MouseEventArgs e)
        {
            fMain.ActiveForm.Cursor = Cursors.Default;
        }

        private void selectClear()
        {
            drawResizeGrabs();
            axImageBox1.Rectangle(x1, y1, x2 + 1, y2 + 1);
            status = 0;
            x1 = 0;
            x2 = 0;
            y1 = 0;
            y2 = 0;
            for (int i = 0; i < 8; i++)
                resizeGrabs[i].active = false;
        }

        private void calcResizeGrabs()
        {
            // Top left
            resizeGrabs[0].x = x1;
            resizeGrabs[0].y = y1;
            resizeGrabs[0].active = true;

            // Top right
            resizeGrabs[1].x = x2;
            resizeGrabs[1].y = y1;
            resizeGrabs[1].active = true;

            // Bottom left
            resizeGrabs[2].x = x1;
            resizeGrabs[2].y = y2;
            resizeGrabs[2].active = true;

            // Bottom right
            resizeGrabs[3].x = x2;
            resizeGrabs[3].y = y2;
            resizeGrabs[3].active = true;

            if (Math.Abs(x2 - x1) > 15)
            {
                // Top
                resizeGrabs[4].x = (x1 + x2) / 2;
                resizeGrabs[4].y = y1;
                resizeGrabs[4].active = true;

                // Bottom
                resizeGrabs[5].x = (x1 + x2) / 2;
                resizeGrabs[5].y = y2;
                resizeGrabs[5].active = true;
            }
            else
            {
                resizeGrabs[4].active = false;
                resizeGrabs[5].active = false;
            }

            if (Math.Abs(y2 - y1) > 15)
            {
                // Left
                resizeGrabs[6].x = x1;
                resizeGrabs[6].y = (y1 + y2) / 2;
                resizeGrabs[6].active = true;

                // Right
                resizeGrabs[7].x = x2;
                resizeGrabs[7].y = (y1 + y2) / 2;
                resizeGrabs[7].active = true;
            }
            else
            {
                resizeGrabs[6].active = false;
                resizeGrabs[7].active = false;
            }
        }

        private void drawResizeGrabs()
        {
            csXImageTrial.TxPenStyle oldPenStyle;
            oldPenStyle = axImageBox1.PenStyle;
            axImageBox1.PenStyle = csXImageTrial.TxPenStyle.psSolid;
            for (int i = 0; i < 8; i++)
            {
                if (resizeGrabs[i].active)
                {
                    axImageBox1.Rectangle(resizeGrabs[i].x - 1, resizeGrabs[i].y - 1, resizeGrabs[i].x + 2, resizeGrabs[i].y + 2);
                    axImageBox1.Rectangle(resizeGrabs[i].x - 2, resizeGrabs[i].y - 2, resizeGrabs[i].x + 3, resizeGrabs[i].y + 3);
                }
            }
            axImageBox1.PenStyle = oldPenStyle;
        }

    }
}
